@@ -125,27 +125,16 @@ static void parse_re2_options(RE2::Options* re2_options, const VALUE options) {
   }
 }
 
-/* For compatibility with Ruby < 2.7 */
-#ifdef HAVE_RB_GC_MARK_MOVABLE
-#define re2_compact_callback(x) (x),
-#else
-#define rb_gc_mark_movable(x) rb_gc_mark(x)
-#define re2_compact_callback(x)
-#endif
-
 static void re2_matchdata_mark(void *ptr) {
   re2_matchdata *m = reinterpret_cast<re2_matchdata *>(ptr);
   rb_gc_mark_movable(m->regexp);
-  rb_gc_mark_movable(m->text);
+  rb_gc_mark(m->text);
 }
 
-#ifdef HAVE_RB_GC_MARK_MOVABLE
 static void re2_matchdata_compact(void *ptr) {
   re2_matchdata *m = reinterpret_cast<re2_matchdata *>(ptr);
   m->regexp = rb_gc_location(m->regexp);
-  m->text = rb_gc_location(m->text);
 }
-#endif
 
 static void re2_matchdata_free(void *ptr) {
   re2_matchdata *m = reinterpret_cast<re2_matchdata *>(ptr);
@@ -171,7 +160,7 @@ static const rb_data_type_t re2_matchdata_data_type = {
     re2_matchdata_mark,
     re2_matchdata_free,
     re2_matchdata_memsize,
-    re2_compact_callback(re2_matchdata_compact)
+    re2_matchdata_compact
   },
   0,
   0,
@@ -183,16 +172,13 @@ static const rb_data_type_t re2_matchdata_data_type = {
 static void re2_scanner_mark(void *ptr) {
   re2_scanner *s = reinterpret_cast<re2_scanner *>(ptr);
   rb_gc_mark_movable(s->regexp);
-  rb_gc_mark_movable(s->text);
+  rb_gc_mark(s->text);
 }
 
-#ifdef HAVE_RB_GC_MARK_MOVABLE
 static void re2_scanner_compact(void *ptr) {
   re2_scanner *s = reinterpret_cast<re2_scanner *>(ptr);
   s->regexp = rb_gc_location(s->regexp);
-  s->text = rb_gc_location(s->text);
 }
-#endif
 
 static void re2_scanner_free(void *ptr) {
   re2_scanner *s = reinterpret_cast<re2_scanner *>(ptr);
@@ -218,7 +204,7 @@ static const rb_data_type_t re2_scanner_data_type = {
     re2_scanner_mark,
     re2_scanner_free,
     re2_scanner_memsize,
-    re2_compact_callback(re2_scanner_compact)
+    re2_scanner_compact
   },
   0,
   0,
@@ -290,10 +276,12 @@ static VALUE re2_matchdata_string(const VALUE self) {
 }
 
 /*
- * Returns the text supplied when incrementally matching with
+ * Returns a frozen copy of the text supplied when incrementally matching with
  * {RE2::Regexp#scan}.
  *
- * @return [String] the original string passed to {RE2::Regexp#scan}
+ * If the text was already a frozen string, returns the original.
+ *
+ * @return [String] a frozen string with the text passed to {RE2::Regexp#scan}
  * @example
  *   c = RE2::Regexp.new('(\d+)').scan("foo")
  *   c.string #=> "foo"
@@ -338,6 +326,11 @@ static VALUE re2_scanner_rewind(VALUE self) {
   delete c->input;
   c->input = new(std::nothrow) re2::StringPiece(
       RSTRING_PTR(c->text), RSTRING_LEN(c->text));
+  if (c->input == 0) {
+    rb_raise(rb_eNoMemError,
+             "not enough memory to allocate StringPiece for input");
+  }
+
   c->eof = false;
 
   return self;
@@ -424,8 +417,8 @@ static re2::StringPiece *re2_matchdata_find_match(VALUE idx, const VALUE self) {
 
   int id;
 
-  if (FIXNUM_P(idx)) {
-    id = FIX2INT(idx);
+  if (RB_INTEGER_TYPE_P(idx)) {
+    id = NUM2INT(idx);
   } else if (SYMBOL_P(idx)) {
     const std::map<std::string, int>& groups = p->pattern->NamedCapturingGroups();
     std::map<std::string, int>::const_iterator search = groups.find(rb_id2name(SYM2ID(idx)));
@@ -693,10 +686,10 @@ static VALUE re2_matchdata_aref(int argc, VALUE *argv, const VALUE self) {
         std::string(RSTRING_PTR(idx), RSTRING_LEN(idx)), self);
   } else if (SYMBOL_P(idx)) {
     return re2_matchdata_named_match(rb_id2name(SYM2ID(idx)), self);
-  } else if (!NIL_P(rest) || !FIXNUM_P(idx) || FIX2INT(idx) < 0) {
+  } else if (!NIL_P(rest) || !RB_INTEGER_TYPE_P(idx) || NUM2INT(idx) < 0) {
     return rb_ary_aref(argc, argv, re2_matchdata_to_a(self));
   } else {
-    return re2_matchdata_nth_match(FIX2INT(idx), self);
+    return re2_matchdata_nth_match(NUM2INT(idx), self);
   }
 }
 
@@ -743,8 +736,9 @@ static VALUE re2_matchdata_inspect(const VALUE self) {
     if (match == Qnil) {
       output << "nil";
     } else {
-      output << "\"" << re2::StringPiece(RSTRING_PTR(match),
-          RSTRING_LEN(match)) << "\"";
+      output << "\"";
+      output.write(RSTRING_PTR(match), RSTRING_LEN(match));
+      output << "\"";
     }
   }
 
@@ -1432,7 +1426,7 @@ static VALUE re2_regexp_match(int argc, VALUE *argv, const VALUE self) {
   RE2::Anchor anchor = RE2::UNANCHORED;
 
   if (RTEST(options)) {
-    if (FIXNUM_P(options)) {
+    if (RB_INTEGER_TYPE_P(options)) {
       n = NUM2INT(options);
 
       if (n < 0) {
@@ -1446,8 +1440,6 @@ static VALUE re2_regexp_match(int argc, VALUE *argv, const VALUE self) {
       VALUE endpos_option = rb_hash_aref(options, ID2SYM(id_endpos));
       if (!NIL_P(endpos_option)) {
 #ifdef HAVE_ENDPOS_ARGUMENT
-        Check_Type(endpos_option, T_FIXNUM);
-
         endpos = NUM2INT(endpos_option);
 
         if (endpos < 0) {
@@ -1476,8 +1468,6 @@ static VALUE re2_regexp_match(int argc, VALUE *argv, const VALUE self) {
 
       VALUE submatches_option = rb_hash_aref(options, ID2SYM(id_submatches));
       if (!NIL_P(submatches_option)) {
-        Check_Type(submatches_option, T_FIXNUM);
-
         n = NUM2INT(submatches_option);
 
         if (n < 0) {
@@ -1493,8 +1483,6 @@ static VALUE re2_regexp_match(int argc, VALUE *argv, const VALUE self) {
 
       VALUE startpos_option = rb_hash_aref(options, ID2SYM(id_startpos));
       if (!NIL_P(startpos_option)) {
-        Check_Type(startpos_option, T_FIXNUM);
-
         startpos = NUM2INT(startpos_option);
 
         if (startpos < 0) {
@@ -1526,37 +1514,43 @@ static VALUE re2_regexp_match(int argc, VALUE *argv, const VALUE self) {
 #endif
     return BOOL2RUBY(matched);
   } else {
+    if (n == INT_MAX) {
+      rb_raise(rb_eRangeError, "number of matches should be < %d", INT_MAX);
+    }
+
     /* Because match returns the whole match as well. */
     n += 1;
 
-    VALUE matchdata = rb_class_new_instance(0, 0, re2_cMatchData);
-    TypedData_Get_Struct(matchdata, re2_matchdata, &re2_matchdata_data_type, m);
-    m->matches = new(std::nothrow) re2::StringPiece[n];
-    RB_OBJ_WRITE(matchdata, &m->regexp, self);
-    if (!RTEST(rb_obj_frozen_p(text))) {
-      text = rb_str_freeze(rb_str_dup(text));
-    }
-    RB_OBJ_WRITE(matchdata, &m->text, text);
-
-    if (m->matches == 0) {
+    re2::StringPiece *matches = new(std::nothrow) re2::StringPiece[n];
+    if (matches == 0) {
       rb_raise(rb_eNoMemError,
                "not enough memory to allocate StringPieces for matches");
     }
 
-    m->number_of_matches = n;
+    text = rb_str_new_frozen(text);
 
 #ifdef HAVE_ENDPOS_ARGUMENT
     bool matched = p->pattern->Match(
-        re2::StringPiece(RSTRING_PTR(m->text), RSTRING_LEN(m->text)),
-        startpos, endpos, anchor, m->matches, n);
+        re2::StringPiece(RSTRING_PTR(text), RSTRING_LEN(text)),
+        startpos, endpos, anchor, matches, n);
 #else
     bool matched = p->pattern->Match(
-        re2::StringPiece(RSTRING_PTR(m->text), RSTRING_LEN(m->text)),
-        startpos, anchor, m->matches, n);
+        re2::StringPiece(RSTRING_PTR(text), RSTRING_LEN(text)),
+        startpos, anchor, matches, n);
 #endif
     if (matched) {
+      VALUE matchdata = rb_class_new_instance(0, 0, re2_cMatchData);
+      TypedData_Get_Struct(matchdata, re2_matchdata, &re2_matchdata_data_type, m);
+
+      RB_OBJ_WRITE(matchdata, &m->regexp, self);
+      RB_OBJ_WRITE(matchdata, &m->text, text);
+      m->matches = matches;
+      m->number_of_matches = n;
+
       return matchdata;
     } else {
+      delete[] matches;
+
       return Qnil;
     }
   }
@@ -1625,10 +1619,14 @@ static VALUE re2_regexp_scan(const VALUE self, VALUE text) {
   VALUE scanner = rb_class_new_instance(0, 0, re2_cScanner);
   TypedData_Get_Struct(scanner, re2_scanner, &re2_scanner_data_type, c);
 
-  c->input = new(std::nothrow) re2::StringPiece(
-      RSTRING_PTR(text), RSTRING_LEN(text));
   RB_OBJ_WRITE(scanner, &c->regexp, self);
-  RB_OBJ_WRITE(scanner, &c->text, text);
+  RB_OBJ_WRITE(scanner, &c->text, rb_str_new_frozen(text));
+  c->input = new(std::nothrow) re2::StringPiece(
+      RSTRING_PTR(c->text), RSTRING_LEN(c->text));
+  if (c->input == 0) {
+    rb_raise(rb_eNoMemError,
+             "not enough memory to allocate StringPiece for input");
+  }
 
   if (p->pattern->ok()) {
     c->number_of_capturing_groups = p->pattern->NumberOfCapturingGroups();
@@ -1924,21 +1922,19 @@ static VALUE re2_set_add(VALUE self, VALUE pattern) {
   re2_set *s;
   TypedData_Get_Struct(self, re2_set, &re2_set_data_type, s);
 
-  /* To prevent the memory of the err string leaking when we call rb_raise,
-   * take a copy of it and let it go out of scope.
-   */
-  char msg[100];
   int index;
+  VALUE msg;
 
   {
     std::string err;
     index = s->set->Add(
         re2::StringPiece(RSTRING_PTR(pattern), RSTRING_LEN(pattern)), &err);
-    strlcpy(msg, err.c_str(), sizeof(msg));
+    msg = rb_str_new(err.data(), err.size());
   }
 
   if (index < 0) {
-    rb_raise(rb_eArgError, "str rejected by RE2::Set->Add(): %s", msg);
+    rb_raise(rb_eArgError,
+             "str rejected by RE2::Set->Add(): %s", RSTRING_PTR(msg));
   }
 
   return INT2FIX(index);
@@ -1961,6 +1957,26 @@ static VALUE re2_set_compile(VALUE self) {
   return BOOL2RUBY(s->set->Compile());
 }
 
+/*
+ * Returns the size of the {RE2::Set}.
+ *
+ * @return [Integer] the number of patterns in the set
+ * @example
+ *   set = RE2::Set.new
+ *   set.add("abc")
+ *   set.size #=> 1
+ */
+static VALUE re2_set_size(VALUE self) {
+#ifdef HAVE_SET_SIZE
+  re2_set *s;
+  TypedData_Get_Struct(self, re2_set, &re2_set_data_type, s);
+
+  return INT2FIX(s->set->Size());
+#else
+  rb_raise(re2_eSetUnsupportedError, "current version of RE2::Set does not have Size method");
+#endif
+}
+
 /*
  * Returns whether the underlying RE2 version outputs error information from
  * {https://github.com/google/re2/blob/bc0faab533e2b27b85b8ad312abf061e33ed6b5d/re2/set.h#L62-L65
@@ -1977,6 +1993,19 @@ static VALUE re2_set_match_raises_errors_p(VALUE) {
 #endif
 }
 
+/*
+ * Returns whether the underlying RE2 version has a Set::Size method.
+ *
+ * @return [Boolean] whether the underlying RE2 has a Set::Size method
+ */
+static VALUE re2_set_size_p(VALUE) {
+#ifdef HAVE_SET_SIZE
+  return Qtrue;
+#else
+  return Qfalse;
+#endif
+}
+
 /*
  * Matches the given text against patterns in the set, returning an array of
  * integer indices of the matching patterns if matched or an empty array if
@@ -2207,11 +2236,15 @@ extern "C" void Init_re2(void) {
 
   rb_define_singleton_method(re2_cSet, "match_raises_errors?",
       RUBY_METHOD_FUNC(re2_set_match_raises_errors_p), 0);
+  rb_define_singleton_method(re2_cSet, "size?",
+      RUBY_METHOD_FUNC(re2_set_size_p), 0);
   rb_define_method(re2_cSet, "initialize",
       RUBY_METHOD_FUNC(re2_set_initialize), -1);
   rb_define_method(re2_cSet, "add", RUBY_METHOD_FUNC(re2_set_add), 1);
   rb_define_method(re2_cSet, "compile", RUBY_METHOD_FUNC(re2_set_compile), 0);
   rb_define_method(re2_cSet, "match", RUBY_METHOD_FUNC(re2_set_match), -1);
+  rb_define_method(re2_cSet, "size", RUBY_METHOD_FUNC(re2_set_size), 0);
+  rb_define_method(re2_cSet, "length", RUBY_METHOD_FUNC(re2_set_size), 0);
 
   rb_define_module_function(re2_mRE2, "Replace",
       RUBY_METHOD_FUNC(re2_Replace), 3);
